{% load i18n %}
{% extends "base.html" %}

{% block content %}
    <h2>{% trans 'Fontes Externas' %}</h2>
    <h3>{% trans 'Fontes Sincronizadas recentemente' %}</h3>
    {% comment %}
        refatorar para remover código jinja
        {% from "dge/macros.jinja" import grid_fonte_externa %}
        {{ grid_fonte_externa(fontes) }}
    {% endcomment %}
{% endblock %}